/* GeoStats - IRC Statistical Services
** Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** GeoStats CVS Identification
** $Id: main.c,v 1.11 2000/09/03 23:11:51 wcampbel Exp $
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
** See the file called 'LICENSE' for more details.
*/

#include "stats.h"

char s_StatServ[MAXNICK] = "StatServ";
char s_Spam[8] = "cm-i";
const char version[] = "Stats $Revision: 1.11 $";
const char version_date[] = __DATE__;
const char version_time[] = __TIME__;

static void start();

int main()
{
	int forked = 0;
	FILE *fp;

	log("Statistics Started (%s).", version);
	me.t_start = time(NULL);
	me.want_privmsg = 0;
	me.enable_spam = 0;

	InitDefaults();
	ConfLoad();

	/* Sanity checking for FLOOD */
	if (me.flood == 1) {
		me.flood = 2;
	}

	/* This should ONLY be run once */
	srand(time(NULL)); 

	LoadStats();
	LoadTLD();
	LoadMyUsers();

#ifndef DEBUG
	forked=fork();
#endif
	if (forked) {
		fp = fopen(PIDFILE, "w");
		fprintf(fp, "%i", forked);
		fclose(fp);
		return 0;
	}

	start();

	return 1;
}

void start()
{
	TimerReset();
	me.users = me.opers = me.servers = me.collisions = 0;
	init_server_hash();
	init_user_hash();
	init_tld();

	log("Connecting to %s:%d", me.uplink, me.port);
	if (servsock > 0)
		close(servsock);

	servsock = ConnectTo(me.uplink, me.port);

	if (servsock <= 0) {
		log("Unable to connect to %s", me.uplink);
	} else {
		login();
		read_loop();
	}
	log("Reconnecting to the server in %d seconds", me.r_time);
	sleep(me.r_time);
	start();
}

/* This is ugly, logging into each server type is slightly different */
void login()
{
   if (!strcmp(me.ircd_type, "hybrid")) {
	sts("PASS %s :TS", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
   } else if (!strcmp(me.ircd_type, "dal") ||
	      !strcmp(me.ircd_type, "mln")) {
	sts("PASS %s", me.pass);
	sts("SERVER %s 1 :%s", me.name, me.desc);
   } else if (!strcmp(me.ircd_type, "hybrid6")) {
	sts("PASS %s :TS", me.pass);
	sts("CAPAB :QS");
	sts("SERVER %s 1 :%s", me.name, me.desc);
   }
}

/* Init the StatServ client */
void init_statserv()
{
   if (!strcmp(me.ircd_type, "hybrid") ||
       !strcmp(me.ircd_type, "hybrid6")) {
	sts("NICK %s 1 1 + %s %s %s :/msg %s \2HELP\2", s_StatServ,
		StatServ.user, StatServ.host, me.name, s_StatServ);
   } else if (!strcmp(me.ircd_type, "dal") ||
	      !strcmp(me.ircd_type, "mln")) {
	sts("NICK %s 1 1 %s %s %s 0 :/msg %s \2HELP\2", s_StatServ,
		StatServ.user, StatServ.host, me.name, s_StatServ);
   }
	AddUser(s_StatServ, StatServ.user, StatServ.host, me.name);
}

/* Bring up the Spam detecting client if it was enabled */
void init_spam()
{
	if (!me.enable_spam)
		return;

   if (!strcmp(me.ircd_type, "hybrid") ||
       !strcmp(me.ircd_type, "hybrid6")) {
	sts("NICK %s 1 1 + please %s %s :Spam Me!", s_Spam,
		me.name, me.name);
   } else if (!strcmp(me.ircd_type, "dal") ||
	      !strcmp(me.ircd_type, "mln")) {
	sts("NICK %s 1 1 please %s %s :Spam Me!", s_Spam,
		me.name, me.name);
   }
	AddUser(s_Spam, "please", me.name, me.name);
}

/* Join the channel and obtain ops if it is a new channel
** We don't take over channels with Stats
*/
void init_join()
{
   char *c;
   if (me.join_chans) {
	for (c = strtok(me.join_chans, ",");
	     c;
	     c = strtok(NULL, ",")) {
	     	if (!strcmp(me.ircd_type, "hybrid") ||
       		    !strcmp(me.ircd_type, "hybrid6")) {
			sts(":%s SJOIN %ld %s + :@%s", me.name, 
			    time(NULL), c, s_StatServ);
		} else if (!strcmp(me.ircd_type, "dal")) {
			sts(":%s JOIN %s", s_StatServ, c);
		} else if (!strcmp(me.ircd_type, "mln")) {
			sts(":%s JOIN %s", s_StatServ, c);
			sts(":%s MODE %s +o %s %ld", me.name, c,
			s_StatServ, time(NULL));
		}
	}
   }
}

/* our own malloc with logging and sanity checking */
void *smalloc(long size)
{
	void *buf;

	if (!size) {
		log("smalloc(): illegal attempt to allocate 0 bytes!");
		size = 1;
	}
	buf = malloc(size);
	if (!buf) {
		log("smalloc(): out of memory.");
		exit(0);
	}
	return buf;
}

/* our own strdup with logging and sanity checking */
char *sstrdup(const char *s)
{
	char *t = strdup(s);
	if (!t) {
		log("sstrdup(): out of memory.");
		exit(0);
	}
	return t;
}

/* Generate a hash value into the tables */
unsigned long HASH(const unsigned char *name, int size_of_table)
{
	unsigned long h = 0, g;

	while (*name) {
		h = (h << 4) + *name++;
			if ((g = h & 0xF0000000))
				h ^= g >> 24;
			h &= ~g;
	}
	return h % size_of_table;
}

/* Lowercase a string */
char *strlower(char *s)
{
	char *t = s;
	while ((*t++ = tolower(*t)))
		;
	return s;
}
